package com.yixin.ms.server;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.QrCodeEnum;
import com.yixin.ms.server.dto.JiliangApartmentDTO;
import com.yixin.ms.server.dto.JiliangCanPassDTO;
import com.yixin.ms.server.dto.JiliangCanPassDataDTO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.JiliangCanPassVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @author: lileilei
 * @date: 2022/8/25 15:53
 * @description: 计量大学校园平台开放接口服务
 */
@Slf4j
@Component
public class JiliangUniversityServer {
    /**
     * token 缓存标识
     */
    private static final String TOKEN = "jiliang_token";

    @Value("${jiliang.appKey}")
    private String appKey;
    @Value("${jiliang.appSecret}")
    private String appSecret;
    @Value("${jiliang.getTokenUrl}")
    private String getTokenUrl;
    @Value("${jiliang.canPassUrl}")
    private String canPassUrl;

    @Value("${jiliang.bcyyCanPassUrl}")
    private String bcyyCanPassUrl;

    @Value("${jiliang.checkIsSchoolBusUrl}")
    private String checkIsSchoolBusUrl;

    @Value("${jiliang.getBcyyTokenUrl}")
    private String getBcyyTokenUrl;

    //公寓 查看学生是否属于该公寓
    @Value("${jiliang.apartment.applyId}")
    private String apartmentApplyId;
    @Value("${jiliang.apartment.secretKey}")
    private String apartmentSecretKey;
    @Value("${jiliang.apartment.url}")
    private String apartmentUrl;

    /**
     * 请求密钥有效时间为 一天 12H
     *
     * @see <a href="https://www.hutool.cn/docs/#/cache/TimedCache">定时缓存</a>
     */
    private static TimedCache<String, String> tokenCache = CacheUtil.newTimedCache(12 * 60 * 60 * 1000);

    /**
     * 获取accesskey（有效期一天）
     *
     * @return
     */
    public String getToken() {
        String token = tokenCache.get(TOKEN, false);
        if (StrUtil.isNotEmpty(token)) {
            return token;
        }

        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("appKey", appKey);
        paramMap.put("appSecret", appSecret);
        HealthResultVO resultVO = null;
        try {
            String body = HttpUtil.post(getTokenUrl, paramMap);
            log.info("计量大学开放平台获取token响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                tokenCache.put(TOKEN, resultVO.getData().toString());
            }
        } catch (Exception e) {
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }

        return resultVO.getData().toString();
    }

    public String getBcyyToken() {

        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("appKey", appKey);
        paramMap.put("appSecret", appSecret);
        HealthResultVO resultVO = null;
        try {
            String body = HttpUtil.post(getBcyyTokenUrl, paramMap);
            log.info("计量大学开放平台获取token响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
        } catch (Exception e) {
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }

        return resultVO.getData().toString();
    }

    /**
     * 获取通过结果 0-不通过，1-通过
     *
     * @param dto
     * @return
     */
    public JiliangCanPassVO canPass(JiliangCanPassDataDTO dto) {
        log.info("--->计量大学开放平台请求参数：{}", JSONObject.toJSONString(dto));
        Map<String, String> headers = Maps.newHashMap();
        headers.put("x-access-token", this.getToken());
        HealthResultVO<JiliangCanPassVO> resultVO = null;
        try {
            String body = HttpUtil.createPost(canPassUrl).addHeaders(headers).body(JSON.toJSONString(new JiliangCanPassDTO().setData(dto))).execute().body();
            log.info("--->计量大学开放平台获取通过结果响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                resultVO.setData(JSON.parseObject(JSON.toJSONString(resultVO.getData()), JiliangCanPassVO.class));
            }
        } catch (Exception e) {
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }
        JiliangCanPassVO data = resultVO.getData();
        if (data == null) {
            throw new ServiceException("计量大学校开放接口服务返回数据格式错误，请联系工作人员");
        }
        return data;
    }

    /**
     * 获取校车系统通过结果 0-不通过，1-通过
     *
     * @param dto
     * @return
     */
    public JiliangCanPassVO bcyyCanPass(JiliangCanPassDataDTO dto) {
        log.info("--->计量大学校车系统请求参数：{}", JSONObject.toJSONString(dto));
        Map<String, String> headers = Maps.newHashMap();
        headers.put("x-access-token", this.getBcyyToken());
        HealthResultVO<JiliangCanPassVO> resultVO = null;
        try {
            String body = HttpUtil.createPost(bcyyCanPassUrl).addHeaders(headers).body(JSON.toJSONString(new JiliangCanPassDTO().setData(dto))).execute().body();
            log.info("--->计量大学校车系统获取通过结果响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                resultVO.setData(JSON.parseObject(JSON.toJSONString(resultVO.getData()), JiliangCanPassVO.class));
            }
        } catch (Exception e) {
            throw new ServiceException("计量大学校车系统接口服务异常，请联系工作人员");
        }

        JiliangCanPassVO data = resultVO.getData();
        if (data == null) {
            throw new ServiceException("计量大学校车系统接口服务返回数据格式错误，请联系工作人员");
        }
        return data;

    }

    /**
     * 学生是否属于该公寓
     *
     * @param dto
     */
    public String schoolApartment(JiliangApartmentDTO dto) {
        log.info("--->计量大学学生是否属于该公寓系统请求参数：{}", JSONObject.toJSONString(dto));
        try {
            HttpClient httpClient = HttpClientBuilder.create().build();
            //设置http请求的连接超时时间
            RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(5000).build();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("page", 1);
            jsonObject.put("pagesize", 10);
//            JSONObject orderParams = new JSONObject();
//            orderParams.put("AGE", "DESC");
//            jsonObject.put("order", orderParams);
            //orderParams 为排序参数,根据传入的排序字段与排序规则对数据进行排序
            JSONObject params = new JSONObject();
            params.put("SCHOOL_CODE", dto.getSchoolCode());
            params.put("CAMPUS_CODE", dto.getSchoolCampusCode());
            params.put("BUILDING_CODE", dto.getSchoolBuildingCode());
            params.put("LH", dto.getLh());
            params.put("XGH", dto.getXgh());
            jsonObject.put("params", params);
            log.info("--->最终请求参数：{}", jsonObject.toString());
            StringEntity entity = new StringEntity(jsonObject.toString(), "UTF-8");
            entity.setContentEncoding("UTF-8");
            entity.setContentType("application/json");
            HttpPost method = new HttpPost(apartmentUrl);
            method.setEntity(entity);
            //密钥传递applyId、secretKey
            method.setHeader("applyId", apartmentApplyId);
            method.setHeader("secretKey", apartmentSecretKey);
            //设置请求参数【超时时间】
            method.setConfig(requestConfig);
            HttpResponse result = httpClient.execute(method);
            HttpEntity ret = result.getEntity();
            String retStr = EntityUtils.toString(ret);
            log.info("--->计量大学学生是否属于该公寓系统响应内容：{}", retStr);
            return retStr;
        } catch (Exception e) {
            throw new ServiceException("计量大学学生公寓系统接口服务异常，请联系工作人员");
        }
    }


    public Boolean checkIsSchoolBus(String sn) {
        Boolean result = Boolean.FALSE;
        try {
            String body = HttpUtil.createGet(checkIsSchoolBusUrl.concat("?sn=" + sn)).execute().body();
            log.info("--->计量大学开放平台获取通过结果响应内容：{}", body);
            JSONObject jsonObject = JSON.parseObject(body);
            if (jsonObject.getInteger("code").equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                result = jsonObject.getBoolean("data");
            }
        } catch (Exception e) {
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }
        return result;
    }

    public static void main(String[] args) {
        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("appKey", "SnNOJW1cx");
        paramMap.put("appSecret", "43f97444e459668c3e19e7bef47491873ba15477");
        HealthResultVO resultVO = null;
        try {
            String body = HttpUtil.post("https://test-api.teamcube.cn/teamcube-coop/app-platform/api/rest/api_token", paramMap);
            log.info("计量大学开放平台获取token响应内容：{}", body);
            resultVO = JSON.parseObject(body, HealthResultVO.class);
            if (resultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                tokenCache.put(TOKEN, resultVO.getData().toString());
            }
        } catch (Exception e) {
            throw new ServiceException("学校开放接口服务异常，请联系工作人员");
        }
        System.out.println(resultVO.getData().toString());
    }


}
